'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getScrapers;

var _fp = require('lodash/fp');

var _fp2 = _interopRequireDefault(_fp);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _filesParser = require('./lib/filesParser.js');

var _filesParser2 = _interopRequireDefault(_filesParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const ONE_HOUR = 3600000;

const initializeScraper = scraper => {
  const scraperInstance = scraper();

  scraperInstance.construct = scraper;

  // eslint-disable-next-line no-param-reassign
  scraperInstance.timeBetween = scraperInstance.timeBetween || ONE_HOUR;

  if (!scraperInstance.name) {
    throw new Error('name must exist');
  }

  if (typeof scraperInstance.timeBetween !== 'number') {
    throw new Error('timeBetween must be a number');
  }

  /* eslint-disable no-param-reassign, no-console */
  scraperInstance.log = (0, _debug2.default)(`scraperjs ${ scraper.name }:log`);
  scraperInstance.warn = (0, _debug2.default)(`scraperjs ${ scraper.name }:warn`);
  scraperInstance.error = (0, _debug2.default)(`scraperjs ${ scraper.name }:error`);
  /* eslint-enable no-param-reassign, no-console */

  return scraperInstance;
};

function getScrapers(files) {
  return (0, _filesParser2.default)(files).then(_fp2.default.map(file => require(file))) // eslint-disable-line global-require
  .then(_fp2.default.map(scraper => scraper.default || scraper)).then(_fp2.default.map(initializeScraper));
}