'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

let ensurePromises = (() => {
  var _ref = _asyncToGenerator(function* (browser) {
    if (process.env.HEADLESS) {
      // phantomjs does not support promises yet (24-05-2016)
      yield browser.executeScript(getSourceForModule('es6-promise'));
    }
  });

  return function ensurePromises(_x) {
    return _ref.apply(this, arguments);
  };
})();

let injectJQuery = (() => {
  var _ref2 = _asyncToGenerator(function* (browser) {
    yield browser.executeScript(`
      var _old$ = window.$;
      var _oldjQuery = window.jQuery;
      ${ getSourceForModule('jquery') };
      window.__SCRAPERJS_JQUERY = window.jQuery;
      window.$ = _old$;
      window.jQuery = _oldjQuery;
    `);
  });

  return function injectJQuery(_x2) {
    return _ref2.apply(this, arguments);
  };
})();

let injectAtoms = (() => {
  var _ref3 = _asyncToGenerator(function* (browser) {
    yield browser.executeScript(`${ atomsSrc }; window.bot = bot;`);
  });

  return function injectAtoms(_x3) {
    return _ref3.apply(this, arguments);
  };
})();

let fixClickHandlers = (() => {
  var _ref4 = _asyncToGenerator(function* (browser) {
    yield browser.executeScript(function () {
      window.__SCRAPERJS_JQUERY.fn.extend({ // eslint-disable-line no-underscore-dangle
        click() {
          bot.action.click(this.get(0));
        }
      });
    });
  });

  return function fixClickHandlers(_x4) {
    return _ref4.apply(this, arguments);
  };
})();

let runWithUtils = (() => {
  var _ref5 = _asyncToGenerator(function* (browser, constructScraper, injectable) {
    const runTheFunction = function (constructScraperStr, utilsStr, callback) {
      const utils = eval(utilsStr); // eslint-disable-line no-eval

      try {
        const fnFromString = function (fnStr) {
          const [, args, body] = fnStr.match(/^function[^(]*\(([^)]*)\) {((?:.|\s)*)}$/);
          return new Function(args.split(','), body); // eslint-disable-line no-new-func
        };

        const scraper = fnFromString(constructScraperStr)();

        let result;

        try {
          result = scraper[utils.queueItem.method](utils); // eslint-disable-line no-undef
        } catch (error) {
          return callback({
            error, from: 'user function', message: error.message
          });
        }

        if (result && result.then && typeof result.then === 'function') {
          const curryCallback = function () {
            return callback({
              queue: utils.queue.getQueue(),
              data: utils.data.getData(),
              finalUrl: window.location.href
            });
          };
          result.then(curryCallback, function (error) {
            return callback({
              error,
              message: error.message,
              from: 'user function promise',
              err: JSON.stringify(error),
              stack: error.stack
            });
          });
          return undefined;
        }

        callback({
          queue: utils.queue.getQueue(),
          data: utils.data.getData(),
          finalUrl: window.location.href
        });
      } catch (error) {
        callback({ error, message: error.message, from: 'runTheFunction code' });
      }
      return undefined;
    };

    return (yield browser.executeAsyncScript(runTheFunction, constructScraper.toString(), injectable.build()))[0];
  });

  return function runWithUtils(_x5, _x6, _x7) {
    return _ref5.apply(this, arguments);
  };
})();

let addToolsToBrowser = (() => {
  var _ref6 = _asyncToGenerator(function* (browser, log) {
    log('ensure promise support');
    yield ensurePromises(browser);
    log('inject jquery');
    yield injectJQuery(browser);
    log('inject atoms');
    yield injectAtoms(browser);
    log('fix jquery click function');
    yield fixClickHandlers(browser);
  });

  return function addToolsToBrowser(_x8, _x9) {
    return _ref6.apply(this, arguments);
  };
})();

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _Injectable = require('./lib/Injectable.js');

var _Injectable2 = _interopRequireDefault(_Injectable);

var _queueUtil = require('./lib/queue-util.js');

var _queueUtil2 = _interopRequireDefault(_queueUtil);

var _dataUtil = require('./lib/data-util.js');

var _dataUtil2 = _interopRequireDefault(_dataUtil);

var _within = require('./lib/within.js');

var _within2 = _interopRequireDefault(_within);

var _during = require('./lib/during.js');

var _during2 = _interopRequireDefault(_during);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /* globals bot */
/* eslint-disable no-underscore-dangle */

const atomsSrc = _fs2.default.readFileSync(_path2.default.join(__dirname, './lib/atoms.js'), 'utf8');

const getSourceForModule = _lodash2.default.memoize(moduleName => _fs2.default.readFileSync(require.resolve(moduleName), 'utf8'));

function constructUtils(queueItem) {
  const injectable = new _Injectable2.default();
  // @todo we might have to white list some fields here...
  injectable.register({ queueItem });
  injectable.register({ queue: _queueUtil2.default });
  injectable.register({ data: _dataUtil2.default });
  injectable.register({ within: _within2.default });
  injectable.register({ during: _during2.default });

  injectable.register({
    $() {
      return window.__SCRAPERJS_JQUERY;
    } });
  return injectable;
}

exports.default = (() => {
  var _ref7 = _asyncToGenerator(function* ({ browser, queueItem, scraper }) {
    let data = [];
    const noop = function (x) {
      return x;
    };
    scraper.log(`navigate to ${ queueItem.url }`);
    yield browser.navigate((scraper.filterUrl || noop)(queueItem.url));

    try {
      yield addToolsToBrowser(browser, scraper.log);
      data = yield runWithUtils(browser, scraper.construct, constructUtils(queueItem));
      return data;
    } catch (err) {
      if (err.message.toString().match(/Document was unloaded during execution/) || err.message.toString().match(/document unloaded while waiting for result/)) {
        scraper.log('retry');
        // page was probably redirected... lets try again?
        try {
          yield addToolsToBrowser(browser, scraper.log);
          yield browser.executeAsyncScript(function (callback) {
            if (document.readyState === 'complete') {
              callback();
            } else {
              window.__SCRAPERJS_JQUERY(window).on('load', function () {
                callback();
              });
            }
          });
          data = yield runWithUtils(browser, scraper[queueItem.method], constructUtils(queueItem));
          return data;
        } catch (errs) {
          scraper.error('BIGERR: An error occurred processing an item', errs, 'from url: ', queueItem.url);
          throw errs;
        }
      }
      scraper.error('BIGERR: An error occurred processing an item', err, 'from url: ', queueItem.url);
      throw err;
    }
  });

  function processBrowser(_x10) {
    return _ref7.apply(this, arguments);
  }

  return processBrowser;
})();