#!/usr/bin/env node
'use strict';

let start = (() => {
  var _ref3 = _asyncToGenerator(function* () {
    const scrapers = yield (0, _getScrapers2.default)(_commander2.default.args);

    function* nextScraper() {
      let i = 0;

      while (true) {
        // eslint-disable-line no-constant-condition
        if (i >= scrapers.length) {
          i = 0;
        }
        log('nextScraper', i);
        yield scrapers[i];
        i += 1;
      }
    }

    yield (0, _promiseEachConcurrency2.default)(nextScraper(), (() => {
      var _ref4 = _asyncToGenerator(function* (scraper) {
        scraper.log('start processing queue');
        try {
          const result = yield startQueue(scraper);
          scraper.log('finished processing queue');
          return result;
        } catch (error) {
          scraper.error('BIGERR: A caught error below');
          scraper.error(error);
          scraper.error(error.stack);
        }
      });

      return function (_x3) {
        return _ref4.apply(this, arguments);
      };
    })(), { concurrency: _commander2.default.concurrency });
  });

  return function start() {
    return _ref3.apply(this, arguments);
  };
})();

var _fp = require('lodash/fp');

var _fp2 = _interopRequireDefault(_fp);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _promiseEachConcurrency = require('promise-each-concurrency');

var _promiseEachConcurrency2 = _interopRequireDefault(_promiseEachConcurrency);

var _cheerio = require('cheerio');

var _cheerio2 = _interopRequireDefault(_cheerio);

var _whacko = require('whacko');

var _whacko2 = _interopRequireDefault(_whacko);

var _browser = require('./lib/browser.js');

var _browser2 = _interopRequireDefault(_browser);

var _getScrapers = require('./getScrapers.js');

var _getScrapers2 = _interopRequireDefault(_getScrapers);

var _process = require('./process.js');

var _process2 = _interopRequireDefault(_process);

var _requireES = require('./lib/requireES6.js');

var _requireES2 = _interopRequireDefault(_requireES);

var _timeout = require('./lib/timeout.js');

var _timeout2 = _interopRequireDefault(_timeout);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const log = (0, _debug2.default)('scraperjs:log');
log.error = (0, _debug2.default)('scraperjs:error');
log.warn = (0, _debug2.default)('scraperjs:warn');

function parseConcurrency(istr) {
  return +istr;
}

_commander2.default.version(require('../package.json').version).option('-q, --queue <file>', 'Use this queue plugin [memory]').option('-d, --data <file>', 'Use this data plugin').option('-c, --concurrency <number>', 'How many browser instances to use', parseConcurrency, 1).parse(process.argv);

if (!_commander2.default.queue || !_commander2.default.data) {
  _commander2.default.help();
}

const dataPlugin = (0, _requireES2.default)(_path2.default.resolve(_commander2.default.data));
const queuePlugin = (0, _requireES2.default)(_path2.default.resolve(_commander2.default.queue));

function startQueue(scraper) {
  return new Promise((() => {
    var _ref = _asyncToGenerator(function* (resolve) {
      let browser;

      const scraperQueue = queuePlugin(`scraperjs:${ scraper.name }`);

      // @todo maybe the scraper queue should be instantiated with expiry
      const addToQueue = function (queueItem) {
        return scraperQueue.add(_fp2.default.assign({ expiry: scraper.timeBetween }, queueItem));
      };

      const filterQueueItems = function (queueItem) {
        if (scraper.filterQueueItem) {
          // eslint-disable-next-line no-param-reassign
          queueItem.url = scraper.filterQueueItem(queueItem);
        }
        return queueItem;
      };

      scraper.log('Add start item to the queue');
      yield addToQueue(filterQueueItems(scraper.start));

      scraper.log('Waiting for 1 seconds');
      yield (0, _timeout2.default)(1000);

      let finishedTimeout;

      function resetFinishTimeout() {
        finishedTimeout = setTimeout(() => {
          console.log('no work in 5 seconds, quitting');
          scraperQueue.close();
          if (browser) {
            browser.quit();
          }
          resolve();
        }, 1000);
      }

      resetFinishTimeout();

      scraperQueue.process({ rateLimit: scraper.rateLimit }, (() => {
        var _ref2 = _asyncToGenerator(function* (queueItem) {
          if (finishedTimeout) {
            clearTimeout(finishedTimeout);
            finishedTimeout = undefined;
          }

          scraper.log('process', queueItem.url);

          if ((queueItem.use || scraper.use || 'browser') === 'browser' && !browser) {
            scraper.log('starting browser');
            browser = yield (0, _browser2.default)();
          }

          const { queue, data, finalUrl } = yield (0, _process2.default)({
            browser,
            queueItem,
            scraper,
            cheerio: _cheerio2.default,
            whacko: _whacko2.default
          }).catch(function (err) {
            scraper.error('error processing item', err);
            throw err;
          });

          scraper.log('finished', queueItem.url);

          scraper.log(queue, data);

          let promises = [];

          if (queue && queue.length) {
            promises = _fp2.default.uniqBy('url')(queue.map(filterQueueItems)).filter(function ({ url }) {
              if (!url.match(/:\/\//)) {
                scraper.error('ignoring invalid url queue attempt', url);
                return false;
              }
              return true;
            }).map(function (_queueItem) {
              return addToQueue(_queueItem);
            });
          }

          if (data) {
            promises.push(dataPlugin({
              url: queueItem.url,
              finalUrl,
              method: queueItem.method,
              scraper: scraper.name,
              timestamp: +new Date(),
              data
            }));
          }

          yield Promise.all(promises);
          resetFinishTimeout();
        });

        return function (_x2) {
          return _ref2.apply(this, arguments);
        };
      })());
    });

    function processQueue(_x) {
      return _ref.apply(this, arguments);
    }

    return processQueue;
  })());
}

start().catch(error => {
  log.error('BIGERR: A caught error', error);
  log.error(error.stack);
});